/**
 * Copyright (c) 1999-2008 Cincom Systems, Inc.
 * US Pat 6,823,384 B1 
 * All Rights Reserved.
 */ 
package Y;

import org.python.core.Py;
import org.python.core.PyObject;
import org.python.core.PyString;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Map;
import java.util.Set;
import java.util.Collection;
import java.util.Vector;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class SurrogatePyDictionaryHash extends ConcurrentHashMap
{
    public Hashtable _shadow;
    public SurrogatePyDictionaryHash( Hashtable shadow ) {
    	super(1, (float)1.0);
        _shadow = shadow;
    }
   
    public void setShadow( Hashtable shadow ) {
        _shadow = shadow;
    }
    public void clear() { _shadow.clear(); }
    public Object clone() { return new SurrogatePyDictionaryHash(_shadow); }
    public boolean containsKey(Object key) {
        if (key instanceof PyString) {
            return _shadow.containsKey(key.toString());
        } else { return false; }
    }
    public Enumeration elements() {
        Enumeration keys;
        Enumeration values;
        int size;
        synchronized ( _shadow ) {
            size    = _shadow.size();
            keys    = _shadow.keys();
            values  = _shadow.elements();
        }
        Vector v = new Vector();
        int i = -1;
        while ( ++i < size ) {
            Object key = keys.nextElement();
            Object val = values.nextElement();
            if ( key instanceof String ) { v.addElement(val); }
        }
        final Enumeration e = v.elements();
        return new Enumeration() {
            public boolean hasMoreElements(){ return e.hasMoreElements(); }
            public Object nextElement(){
                return Py.java2py( e.nextElement() );
            }
        };
    }
    public boolean equals(Object o) { return (this == o); }
    public Object get(Object key) {
        if (key instanceof PyString || key instanceof String) {
            Object val = _shadow.get( key.toString() );
            val = ( val instanceof Boolean ) ? val.toString() : val;
            return Py.java2py( val );
            //return Py.java2py( _shadow.get(key.toString()) );
        } else { return null; }
    }
    // This impl of isEmpty is rather inefficient, but it's not used.
    public boolean isEmpty() {
        return _shadow.isEmpty() || !elements().hasMoreElements();
    }
    public Enumeration keys() {
        Enumeration keys;
        int size;
        synchronized ( _shadow ) {
            size    = _shadow.size();
            keys    = _shadow.keys();
        }
        Vector v = new Vector();
        int i = -1;
        while ( ++i < size ) {
            Object key = keys.nextElement();
            if ( key instanceof String ) { v.addElement(key); }
        }
        final Enumeration e = v.elements();
        return new Enumeration() {
            public boolean hasMoreElements(){ return e.hasMoreElements(); }
            public Object nextElement(){
                return new PyString( (String) e.nextElement() );
            }
        };
    }
    
    public Object put(Object key, Object value) {
        if ( key instanceof PyString || key instanceof String ) {
            return Py.java2py( _shadow.put(
                    key.toString(),
                    Py.tojava((PyObject)value,Object.class)
                )
            );
        } else { throw Py.TypeError("dictionary key must be a string"); }
    }
    public Object remove(Object key) {
        if ( key instanceof PyString || key instanceof String ) {
            return Py.java2py( _shadow.remove(key.toString()) );
        } else { throw Py.TypeError("dictionary key must be a string"); }
    }
    public int size() {
        int shadowSize;
        Enumeration e;
        synchronized ( _shadow ) {
            shadowSize = _shadow.size();
            e = _shadow.keys();
        }
        int size = 0;
        int i = -1;
        while ( ++i < shadowSize ) {
            if (e.nextElement() instanceof String) size++;
        }
        return size;
    }
    // Jython 2.0 PyDictionary does not use the following methods:
    //public boolean contains(Object value) { return false; }
    //public boolean containsValue(Object value)
    //public Set entrySet()
    //public int hashCode()
    //public Set keySet()
    //public void putAll(Map t)
    //protected void rehash()
    //public Collection values()
    
	/*public boolean containsValue(Object value) {
		// TODO Auto-generated method stub
		return getMap().containsValue(Py.java2py(value));
	}
	
	public void putAll(Map map) {
		// TODO Auto-generated method stub
		 for (Object o : map.entrySet()) {
	            Entry entry = (Entry)o;
	            getMap().put(Py.java2py(entry.getKey()), Py.java2py(entry.getValue()));
	      }
	}
	
	public Set keySet() {
		// TODO Auto-generated method stub
		return getMap().keySet();
	}
	
	public Collection values() {
		// TODO Auto-generated method stub
		return getMap().values();
	}
	
	public Set entrySet() {
		// TODO Auto-generated method stub
		return getMap().entrySet();
	}
	
	public Object putIfAbsent(Object key, Object value) {
		// TODO Auto-generated method stub
		return getMap().putIfAbsent(Py.java2py(key), Py.java2py(value));
	}
	

	public boolean replace(Object key, Object oldValue, Object newValue) {
		// TODO Auto-generated method stub
		return getMap().replace(Py.java2py(key), Py.java2py(oldValue), Py.java2py(newValue));
	}
	@Override
	public Object replace(Object key, Object value) {
		// TODO Auto-generated method stub
		return getMap().replace(Py.java2py(key), Py.java2py(value));
	}
	@Override
	public boolean remove(Object key, Object value) {
		// TODO Auto-generated method stub
		return getMap().remove(Py.java2py(key), Py.java2py(value));
	}*/
}
